From ok@artecdesign.ee Thu Apr 21 07:16:03 2005
Date: Thu, 21 Apr 2005 17:12:59 +0300 (EEST)
From: Olav Kongas <ok@artecdesign.ee>
To: Greg KH <greg@kroah.com>
Subject: USB: Fix oops at rmmod after failed probe in isp116x-hcd


This patch fixes an oops triggered at rmmod of isp116x-hcd
after the probe() has failed.

Also, it extends the error message printed, if the driver
cannot detect "Chip's Clock Ready" after a software reset.
As Ian Campbell recently reported, this happens if the
chip's H_WAKEUP pin is not pulled low during software reset.
Several people have already had this issue, hence the update
to the error message.

Also, extend the error message about the failed clock
detection after the software reset.

Signed-off-by: Olav Kongas <ok@artecdesign.ee>

Index: gregkh-2.6/drivers/usb/host/isp116x-hcd.c
===================================================================
--- gregkh-2.6.orig/drivers/usb/host/isp116x-hcd.c	2005-04-22 14:30:46.000000000 -0700
+++ gregkh-2.6/drivers/usb/host/isp116x-hcd.c	2005-04-22 14:37:42.000000000 -0700
@@ -1501,6 +1501,12 @@
 	}
 	if (!clkrdy) {
 		ERR("Clock not ready after 20ms\n");
+		/* After sw_reset the clock won't report to be ready, if
+		   H_WAKEUP pin is high. */
+		if (!isp116x->board || !isp116x->board->reset)
+			ERR("The driver does not support hardware wakeup.\n");
+			ERR("Please make sure that the H_WAKEUP pin "
+				"is pulled low!\n");
 		ret = -ENODEV;
 	}
 	return ret;
@@ -1678,10 +1684,13 @@
 static int __init_or_module isp116x_remove(struct device *dev)
 {
 	struct usb_hcd *hcd = dev_get_drvdata(dev);
-	struct isp116x *isp116x = hcd_to_isp116x(hcd);
+	struct isp116x *isp116x;
 	struct platform_device *pdev;
 	struct resource *res;
 
+	if(!hcd)
+		return 0;
+	isp116x = hcd_to_isp116x(hcd);
 	pdev = container_of(dev, struct platform_device, dev);
 	remove_debug_file(isp116x);
 	usb_remove_hcd(hcd);
